/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "ClientSettings.h"
#include "ClientUtil.h"
#include "spds/constants.h"

#define PROPERTY_PUSH_NOTIFICATION      ("push")
#define PROPERTY_NOTIFICATION_PORT      ("pushPort")
#define PROPERTY_SMS_WAP_PUSH           ("sms")
#define PROPERTY_POLLING_NOTIFICATION   ("polling")
#define PROPERTY_SERVER_NOTIFIED        ("svrNotified")
#define PROPERTY_SOURCE_USE_SIF         ("useSIF") 
#define PROPERTY_SOURCE_DIR             ("dir")
#define PROPERTY_PATH                   ("path")
#define PROPERTY_SCHEDULE               ("schedule")

// to decide if the plug-in version is for portal or not
#define PROPERTY_IS_PORTAL              ("isPortal")


 

ClientSettings::ClientSettings(char* application_uri) : DMTClientConfig(application_uri) {
    setIp(CString(""));
    setPush(CString(""));
    setPushPost(CString(""));
    setSms(CString(""));
    setPolling(CString(""));
    setUseSif(CString(""));
    setDir(CString(""));
    setPath(CString(""));
    setSvrNotified(CString(""));
    setSchedule(CString(""));
    setIsPortal(CString(""));

}

ClientSettings::~ClientSettings() {	
}

void ClientSettings::setIp(CString extip) {
    ipaddress = extip;
}
CString ClientSettings::getIp() {
    return ipaddress;
}

void ClientSettings::setPush(CString extpush) {
    push = extpush;
}
CString ClientSettings::getPush() {
    return push;
}
		
void ClientSettings::setPushPost(CString extpushPort) {
    pushPort = extpushPort;
}
CString ClientSettings::getPushPort() {
    return pushPort;
}

void ClientSettings::setSms(CString extsms){
    sms = extsms;
}
CString ClientSettings::getSms() {
    return sms;
}

void ClientSettings::setPolling(CString extpolling) {
    polling = extpolling;
}
CString ClientSettings::getPolling() {
    return polling;
}   

void ClientSettings::setSvrNotified(CString extsvrNotified) {
    svrNotified = extsvrNotified;
}
CString ClientSettings::getSvrNotified() {
    return svrNotified;
} 

void ClientSettings::setPath(CString extpath) {
    path = extpath;
}
CString ClientSettings::getPath() {
    return path;
}  

// for the sources

void ClientSettings::setUseSif(CString extuseSif) {
    useSif = extuseSif;
}
CString ClientSettings::getUseSif() {
    return useSif;
}

void ClientSettings::setSchedule(CString extSchedule){
    schedule = extSchedule;
}

CString ClientSettings::getSchedule(){
    return schedule;
}

void ClientSettings::setDir(CString extdir) {
    dir = extdir;
}
CString ClientSettings::getDir() {
    return dir;
}

void ClientSettings::setIsPortal(CString extportal) {
    isPortal = extportal;
}
bool ClientSettings::getIsPortal() {
    return isPortal == "1" ? true : false;
}

/**
 * read all the AccessConfig configuration of the client.
 * there is the Acc, Ext, Conn nodes
 */
BOOL ClientSettings::readConfig(BOOL toClose) {
	int n = 0, i = 0; // number of sync sources
    BOOL ret = FALSE;
    LOG.debug("ClientSettings: readConfig");
    char nodeName[DIM_MANAGEMENT_PATH];
    char syncMLContext[DIM_MANAGEMENT_PATH];

    //
    // Reading syncml node
    //
    //char nodeName[DIM_MANAGEMENT_PATH];

    if (!open()) {
        return FALSE;
    }
    
    readAccessConfig(*syncMLNode);
    
    ManagementNode* node;
    //
    // Ext properties (other misc props)
    //
    char* fn = syncMLNode->createFullName();
    sprintf(syncMLContext, "%s", fn);
    delete [] fn;    
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_EXT);
    node = dmt->getManagementNode(nodeName);
    if (node) {
        if (!readExtDevConfig(*syncMLNode, *node)) {
            ret = FALSE;
        }
        delete node; 
        node = NULL;
    }
    else {
        ret = FALSE;
    }
    
    //
    // DevInfo properties
    //
    fn = syncMLNode->createFullName();
    sprintf(syncMLContext, "%s", fn);
    delete [] fn;
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_DEV_INFO);
    node = dmt->getManagementNode(nodeName);
    if (node) {
        if (!readDevInfoConfig(*syncMLNode, *node)) {
            ret = FALSE;
        }
        delete node; 
        node = NULL;
    }
    else {
        ret = FALSE;
    }

    ret = TRUE;
    
    // read parameter of the client only
    
    node = dmt->getManagementNode(rootContext);
    if (node) {
        char* tmp;
        CString t;
        tmp = node->readPropertyValue(PROPERTY_PUSH_NOTIFICATION);
        t = tmp;
        setPush(t);    
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_NOTIFICATION_PORT);
        t = tmp;
        setPushPost(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_SMS_WAP_PUSH);
        t = tmp;
        setSms(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_POLLING_NOTIFICATION);
        t = tmp;
        setPolling(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_SERVER_NOTIFIED);
        t = tmp;
        setSvrNotified(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_PATH);
        t = tmp;
        setPath(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_IS_PORTAL);
        t = tmp;
        setIsPortal(t);
        delete [] tmp;

        delete node; 
        node = NULL;
    }

    if (toClose)
        close();
    return ret;
}

/**
 * It saves only the parameter that can be changed by the configuration
 * in the different Acc, Ext, Conn DevInfo nodes
 * username/password in Auth
 * syncURL in Conn 
 * log/encryption in Ext
 */
BOOL ClientSettings::saveConfig() {
    
    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];

    char syncMLContext[DIM_MANAGEMENT_PATH];
    char* fn = syncMLNode->createFullName();
    sprintf(syncMLContext, "%s", fn);
    delete [] fn;
       
    //
    // Auth properties
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_AUTH);
    node = dmt->getManagementNode(nodeName);
    if (node) {
       
        node->setPropertyValue(PROPERTY_USERNAME, accessConfig.getUsername());
        node->setPropertyValue(PROPERTY_PASSWORD, accessConfig.getPassword());
        
        delete node; 
        node = NULL;
    }
    
    //
    // DevInfo properties
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_DEV_INFO);
    node = dmt->getManagementNode(nodeName);
    if (node) {
               
        node->setPropertyValue(PROPERTY_DEVICE_ID, deviceConfig.getDevID());        

        delete node; 
        node = NULL;
    }


    //
    // Conn properties
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_CONN);
    node = dmt->getManagementNode(nodeName);
    if (node) {
        node->setPropertyValue(PROPERTY_SYNC_URL, accessConfig.getSyncURL());
            
        delete node; 
        node = NULL;
    }

    //
    // Ext properties (other misc props)
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_EXT);
    node = dmt->getManagementNode(nodeName);
    if (node) {
        char buf[512];

        //node->setPropertyValue(PROPERTY_ENCRYPTION,
        //                     (accessConfig.getEncryption() ? T("1") : T("0") ) ); 
        
        sprintf(buf, "%lu", deviceConfig.getLogLevel());        
        node->setPropertyValue(PROPERTY_LOG_LEVEL, buf);
        
        node->setPropertyValue(PROPERTY_DEVINF_HASH, deviceConfig.getDevInfHash());

        delete node; 
        node = NULL;
    }
    
    // read parameter of the client only
    
    node = dmt->getManagementNode(rootContext);    
    if (node) {
        node->setPropertyValue(PROPERTY_PUSH_NOTIFICATION, _wcc(getPush().GetString()));
        node->setPropertyValue(PROPERTY_NOTIFICATION_PORT, _wcc(getPushPort().GetString()));
        node->setPropertyValue(PROPERTY_SMS_WAP_PUSH, _wcc(getSms().GetString()));
        node->setPropertyValue(PROPERTY_POLLING_NOTIFICATION, _wcc(getPolling().GetString()));
        node->setPropertyValue(PROPERTY_SERVER_NOTIFIED, _wcc(getSvrNotified().GetString()));
        node->setPropertyValue(PROPERTY_PATH, _wcc(getPath().GetString()));
        delete node; 
        node = NULL;
    }

    close();
    return TRUE;

}

const char* ClientSettings::getConfigSourcesParameter(const char* sourceName, const char* parameter) { 
        
    for (unsigned int i=0; i<sourceConfigsCount; ++i) {
        if (strcmp(sourceConfigs[i].getName(), sourceName) == 0) {
            if (strcmp(parameter, "sync") == 0)
                return sourceConfigs[i].getSync();            
            else
                return "to implement";            
        }
    }   
    return "";

}

BOOL ClientSettings::setConfigSourcesParameter(const char* sourceName, const char* parameter, const char* value) { 
        
    for (unsigned int i=0; i<sourceConfigsCount; ++i) {
        if (strcmp(sourceConfigs[i].getName(), sourceName) == 0) {
            if (strcmp(parameter, "sync") == 0) {
                sourceConfigs[i].setSync(value); 
            } else if (strcmp(parameter, "last") == 0) {
                sourceConfigs[i].setLast(atoi(value)); 
            }
            return true;
        }
    }   
    return false;
}

BOOL ClientSettings::readConfigSources(const char* sourceType, BOOL toClose) {
    
    int n = 0, i = 0; // number of sync sources

    BOOL ret = FALSE;
    ManagementNode* nn = NULL;

    if (!open()) {
        return FALSE;
    }

    n = sourcesNode->getChildrenMaxCount();

    //
    // Let's remove previously created config objects and reinitialize
    // sourceConfigs
    //
    if (sourceConfigs) {
        delete [] sourceConfigs;
    }
    sourceConfigs = new SyncSourceConfig[n];
    sourceConfigsCount = n;

    for (i=0; i<n; ++i) {
        // node owns children, we must not delete them
        readSourceConfig(i, *(sourcesNode) );
    }
    
    if (sourceType && strcmp(sourceType, "all") == 0) {
         if (toClose)
            close();   
        return true;
    }

    for (unsigned int i=0; i<sourceConfigsCount; ++i) {
        if (strcmp(sourceConfigs[i].getName(), sourceType) == 0) {
            ssconfig.assign(sourceConfigs[i]);
            nn = sourcesNode->getChild(i);
            break;
        }
    }        
    
    char* tmp;

    if (strcmp(sourceType, "contact") == 0) {        
        
        tmp = nn->readPropertyValue(PROPERTY_SOURCE_USE_SIF);    
        setUseSif(CString(tmp));
        delete [] tmp;  
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        setSchedule(CString(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "calendar") == 0) {        
        
        tmp = nn->readPropertyValue(PROPERTY_SOURCE_USE_SIF);    
        setUseSif(CString(tmp));
        delete [] tmp;  
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        setSchedule(CString(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "task") == 0) {        
        
        tmp = nn->readPropertyValue(PROPERTY_SOURCE_USE_SIF);    
        setUseSif(CString(tmp));
        delete [] tmp;                      
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        setSchedule(CString(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "briefcase") == 0) {        
        
        tmp = nn->readPropertyValue(PROPERTY_SOURCE_DIR);    
        setDir(CString(tmp));
        delete [] tmp;  
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        setSchedule(CString(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "note") == 0) {        
        tmp = nn->readPropertyValue(PROPERTY_SOURCE_DIR);    
        setDir(CString(tmp));
        delete [] tmp;  
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        setSchedule(CString(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "mail") == 0) {    
        const char* context = NULL;
        char t[255];
        char* fn = nn->createFullName();
        strcpy(t, fn);
        delete [] fn;

        context = strstr(t, nn->getName());
        t[context - t - 1] = 0;
                
        MailSourceManagementNode* mmn = new MailSourceManagementNode(t, nn->getName());

        mailssconfig = mmn->getMailSourceConfig(true);
              
    }
    ret = TRUE;

    if (toClose)
        close();
    
    return ret;
    
}


BOOL ClientSettings::saveConfigSources(const char* saveAll, bool alsoLast) {
    
    ManagementNode* nn;
    
    // to save all the sources config. It is used by the form
    // in which there are all the sources whith the list of the sync property
    // and to reset the last timestamp in the recover form. it is set to 1
    //
    
    if (saveAll && strcmp(saveAll, "all") == 0) {

        for (unsigned int i=0; i<sourceConfigsCount; ++i) {        
            if (alsoLast) {
                saveSourceConfig(i, *sourcesNode);        
            } else {
                nn = sourcesNode->getChild(i);
                saveSourceConfig(i, *sourcesNode, *nn);        
            }
            
        }   
        close();
        return true;
    }


    for (unsigned int i=0; i<sourceConfigsCount; ++i) {
        
        if (strcmp(sourceConfigs[i].getName(), ssconfig.getName()) == 0) {           
            nn = sourcesNode->getChild(i);
            break;
        }
    }    
    
    if (strcmp(ssconfig.getName(), "contact")  == 0 ||
        strcmp(ssconfig.getName(), "calendar") == 0 ||
        strcmp(ssconfig.getName(), "task")  == 0) {        

        nn->setPropertyValue(PROPERTY_SOURCE_URI, ssconfig.getURI());
        nn->setPropertyValue(PROPERTY_SOURCE_SYNC, ssconfig.getSync());
        nn->setPropertyValue(PROPERTY_SOURCE_USE_SIF, _wcc(getUseSif().GetString()));       
        nn->setPropertyValue(PROPERTY_SCHEDULE, _wcc(getSchedule().GetString()));
        nn->setPropertyValue(PROPERTY_SOURCE_ENCRYPTION, ssconfig.getEncryption());
        //
        // to set the encoding parameter automatically
        //
        if (getUseSif() == "1") {
            nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, ("b64"));
            if (strcmp(ssconfig.getName(), "contact")  == 0) {
                nn->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-s4j-sifc");           
            } else if (strcmp(ssconfig.getName(), "calendar") == 0) {                         
                nn->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-s4j-sife");           
            } else if (strcmp(ssconfig.getName(), "task") == 0)  {                       
                nn->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-s4j-sift");           
            }

        } else {
            if(strcmp(ssconfig.getEncryption(), "0") != 0) {
                nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
            } else {
                nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
            }
            if (strcmp(ssconfig.getName(), "contact")  == 0) {  // 2.1
                nn->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-vcard");           
            } else if (strcmp(ssconfig.getName(), "calendar") == 0 ||
                       strcmp(ssconfig.getName(), "task")  == 0) {        
                nn->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-vcalendar");           
            }
        }

    } else if( (strcmp(ssconfig.getName(), "briefcase") == 0) ||
               (strcmp(ssconfig.getName(), "note") == 0) )
    {        
        
        nn->setPropertyValue(PROPERTY_SOURCE_URI, ssconfig.getURI());
        nn->setPropertyValue(PROPERTY_SOURCE_SYNC, ssconfig.getSync());
        nn->setPropertyValue(PROPERTY_SOURCE_TYPE, ssconfig.getType());
        nn->setPropertyValue(PROPERTY_SOURCE_DIR, _wcc(getDir().GetString()));  
        nn->setPropertyValue(PROPERTY_SCHEDULE, _wcc(getSchedule().GetString()));
        nn->setPropertyValue(PROPERTY_SOURCE_ENCRYPTION, ssconfig.getEncryption());
        /*
        * Note are always in b64. briefcase has the content of File structure in b64
        */
        if(strcmp(ssconfig.getEncryption(), "0") != 0 || strcmp(ssconfig.getName(), "note") == 0) {
            nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
        } else {
            nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
        }
        //nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, ssconfig.getEncoding());

    } else if (strcmp(ssconfig.getName(), "mail") == 0) {    

        //setMailSourceConfig(mailssconfig, nn);
        
        const char* context = NULL;
        char* tt = nn->createFullName();
        char t[255];
        strcpy(t, tt);
        context = strstr(t, nn->getName());
        t[context - t - 1] = 0;
               
        MailSourceManagementNode* mmn = new MailSourceManagementNode(t, nn->getName());
                
        if(strcmp(mailssconfig.getEncryption(), "0") != 0) {
            mailssconfig.setEncoding("b64");
        } else {
            mailssconfig.setEncoding("bin");
        }

        mmn->setMailSourceConfig(mailssconfig);        
        if (tt) { delete [] tt;}
    }
          
    close();
    return TRUE;

}
/*
void ClientSettings::setMailSourceConfig(MailSyncSourceConfig& c, ManagementNode* nn) {   
    char t[512];

    nn->setPropertyValue(PROPERTY_SOURCE_NAME,       (char*)c.getName());
    nn->setPropertyValue(PROPERTY_SOURCE_URI,        (char*)c.getURI());
    nn->setPropertyValue(PROPERTY_SOURCE_SYNC_MODES, (char*)c.getSyncModes());
    nn->setPropertyValue(PROPERTY_SOURCE_SYNC,       (char*)c.getSync());
    nn->setPropertyValue(PROPERTY_SOURCE_TYPE,       (char*)c.getType());

    sprintf(t, T("%d"), c.getDownloadAge());
    nn->setPropertyValue(PROPERTY_SOURCE_DOWNLOAD_AGE, t);
    sprintf(t, T("%d"), c.getBodySize());
    nn->setPropertyValue(PROPERTY_SOURCE_BODY_SIZE, t);
    sprintf(t, T("%d"), c.getAttachSize());
    nn->setPropertyValue(PROPERTY_SOURCE_ATTACH_SIZE, t);

    sprintf(t, T("%d"), c.getInbox());
    nn->setPropertyValue(PROPERTY_SOURCE_INBOX, t);
    sprintf(t, T("%d"), c.getOutbox());
    nn->setPropertyValue(PROPERTY_SOURCE_OUTBOX, t);
    sprintf(t, T("%d"), c.getTrash());
    nn->setPropertyValue(PROPERTY_SOURCE_TRASH, t);
    sprintf(t, T("%d"), c.getSent());
    nn->setPropertyValue(PROPERTY_SOURCE_SENT, t);
    sprintf(t, T("%d"), c.getDraft());
    nn->setPropertyValue(PROPERTY_SOURCE_DRAFT, t);

}
*/