/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// MailSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "MailSettings.h"
#include "MailFoldersSettings.h"
#include "MailFiltersSettings.h"
#include "RegistrySettings.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "comutil.h"
#include "pim/Maincpp.h"
#include "tpcshell.h"  //-- for SHSendBackToFocusWindow

#include "pim/account.h"

#define DEFAULT_USER "default@email"

// CMailSettings dialog

IMPLEMENT_DYNAMIC(CMailSettings, CDialog)

CMailSettings::CMailSettings(CWnd* pParent /*=NULL*/)
	: CDialog(CMailSettings::IDD, pParent)
{

}

CMailSettings::~CMailSettings()
{
}

void CMailSettings::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
    #if defined(WIN32_PLATFORM_WFSP)
        DDX_Control(pDX, IDC_SETTMAIL_SPIN1, spinSyncType );
    #endif
    DDX_Control(pDX, IDC_MAIL_SYNCTYPE, lstSyncType );
    DDX_Control(pDX, IDC_SETTMAIL_EDIT_REMOTE, editRemote );
    DDX_Control(pDX, IDC_EMAIL_EDIT_DISPNAME, editName);
    DDX_Control(pDX, IDC_EMAIL_EDIT_EMAIL, editEmail);
    #if defined(WIN32_PLATFORM_PSPC)
        DDX_Control(pDX, IDC_MAIL_ADVANCED, staticAdvanced);
    #endif
}


BEGIN_MESSAGE_MAP(CMailSettings, CDialog)
    #if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
        ON_WM_SIZE()
    #endif
    ON_COMMAND(ID_MENU_FOLDERS, &CMailSettings::OnMenuFolders)
    ON_COMMAND(ID_MENU_FILTERS, &CMailSettings::OnMenuFilters)
   #ifdef WIN32_PLATFORM_WFSP
     ON_MESSAGE(WM_HOTKEY,&CMailSettings::OnHotkey)
   #endif
END_MESSAGE_MAP()


BOOL CMailSettings::OnInitDialog()
{
    CString s1;
    wchar_t* accountName = new wchar_t[ACCOUNT_PARAM_SIZE];
    wchar_t* replyAddress = new wchar_t[ACCOUNT_PARAM_SIZE];

    //-- get account info
    SetCursor(LoadCursor(NULL, IDC_WAIT));
    HRESULT hr = getAccountInfo(accountName, replyAddress);   //-- this may take a while
    SetCursor(LoadCursor(NULL, IDC_ARROW));

    if (hr == -1) {
        delete [] accountName; delete [] replyAddress;         
        AfxMessageBox(_T("Funambol email account not found."));   
        CDialog::OnCancel();
        return FALSE;
    }

    s1.LoadString(IDS_MAIL_SETTINGS); SetWindowText(s1); 
    CDialog::OnInitDialog(); 

  #if defined(WIN32_PLATFORM_WFSP)
    spinSyncType.SetRange(4,0);	spinSyncType.SetBuddy(&lstSyncType);  

    if (!dlgCommandBar3.Create(this) ||
       !dlgCommandBar3.InsertMenuBar(IDR_MENUBAR_SETTMAIL))
    {
		return FALSE;      // failed to create
    } 
     dlgCommandBar3.Show(true);
  #else
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO); 
    dlgCommandBar.hwndParent = this->GetSafeHwnd(); 
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDR_MENU_SETTMAIL; 
    //dlgCommandBar.nToolBarId = IDM_OKCANCEL; 
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0; 
    dlgCommandBar.cBmpImages = 0; 
    if(!SHCreateMenuBar(&dlgCommandBar)){
        AfxMessageBox(_T("Cannot create command bar!")); 
        return FALSE;
    }
  #endif

  editRemote.SetLimitText(EDIT_MAXLENGTH);
  editName.SetLimitText(EDIT_MAXLENGTH);
  editEmail.SetLimitText(EDIT_MAXLENGTH);

  s1.LoadString(IDS_MAIL_SYNCTYPE1); lstSyncType.AddString(s1);
  s1.LoadString(IDS_MAIL_SYNCTYPE2); lstSyncType.AddString(s1);
  //s1.LoadString(IDS_MAIL_SYNCTYPE3); lstSyncType.AddString(s1);
  //s1.LoadString(IDS_MAIL_SYNCTYPE4); lstSyncType.AddString(s1);

  s1.LoadString(IDS_SYNCTYPE); SetDlgItemText(IDC_SETTMAIL_STATIC_SYNCTYPE,s1);
  s1.LoadString(IDS_REMOTENAME); SetDlgItemText(IDC_SETTMAIL_STATIC_REMONAME,s1);
  s1.LoadString(IDS_SETTMAIL_BUTFOLDERS); SetDlgItemText(IDC_SETTMAIL_BUTFOLDERS,s1);
  s1.LoadString(IDS_SETTMAIL_BUTFILTER); SetDlgItemText(IDC_SETTMAIL_BUTFILTER,s1);
  s1.LoadString(IDS_SETTMAIL_BUTACCOUNT); SetDlgItemText(IDC_SETTMAIL_BUTACCOUNT,s1);

  s1.LoadString(IDS_ACCOUNT_DISPNAME); SetDlgItemText(IDC_ACCOUNT_STATIC_NAME,s1);
  s1.LoadString(IDS_ACCOUNT_EMAIL); SetDlgItemText(IDC_ACCOUNT_STATIC_EMAIL,s1);

  CString sTemp1(accountName), sTemp2(replyAddress);
  SetDlgItemText(IDC_EMAIL_EDIT_DISPNAME,sTemp1);

  dispName = sTemp1; dispEmail = sTemp2;

  if (sTemp2 == DEFAULT_USER) {
    sTemp2 = "";
  } else {
    SetDlgItemText(IDC_EMAIL_EDIT_EMAIL, sTemp2);
  } 

  //----- init values with values read from the registry -------
  SyncRegSettings regsettings("mail");
  CString s3; int pos=0;

  s3 = regsettings.cs.mailssconfig.getURI(); SetDlgItemText(IDC_SETTMAIL_EDIT_REMOTE,s3);
  if(!strcmp(regsettings.cs.mailssconfig.getSync(),"two-way"))
  {
     pos = 0;
  }

  if(!strcmp(regsettings.cs.mailssconfig.getSync(),"refresh-server"))
         pos =1;
  lstSyncType.SetCurSel(pos);

  #ifdef WIN32_PLATFORM_WFSP
    //-- override back key
    LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
	    	  SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
    ::SendMessage(dlgCommandBar3, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
  #endif

  //-- set a large int as current selection, so the cursor will be at end
    /*
  editRemote.SetSel(LARGE_INT ,LARGE_INT);
  editName.SetSel(LARGE_INT ,LARGE_INT);
  editEmail.SetSel(LARGE_INT ,LARGE_INT);
  */
  editRemote.SetSel(0);
  editName.SetSel(0);
  editEmail.SetSel(0);
  
  if(accountName)
      delete [] accountName;
  if(replyAddress)
      delete [] replyAddress;

  lstSyncType.SetFocus();
  return FALSE;
}
// CMailSettings message handlers

void CMailSettings::OnOK()
{
	//-- save values in registry
   SyncRegSettings regsettings("mail", false);
   CString s1, s2; int pos=0;
   _bstr_t bst; 
   bool bSaveAccount = false;
   wchar_t* accountName = NULL; 
   wchar_t* replyAddress = NULL;

   GetDlgItemText(IDC_SETTMAIL_EDIT_REMOTE, s1); 
   bst.Assign(s1.AllocSysString());	
   regsettings.cs.mailssconfig.setURI((char*)bst);
   bst.Detach();

   pos = lstSyncType.GetCurSel(); 
    
    //
    // set type of sync
    // 
    if(pos == 0)
        regsettings.cs.mailssconfig.setSync("two-way");	
    else // pos = 1 
        regsettings.cs.mailssconfig.setSync("refresh-server");	 
    

    //-- save mail account settings
    GetDlgItemText(IDC_EMAIL_EDIT_DISPNAME,s1);
    GetDlgItemText(IDC_EMAIL_EDIT_EMAIL,s2);
    /*
    if (s1.IsEmpty()) {
        AfxMessageBox(_T("The Display Name cannot be empty"));
        return;
    }    
    if (s2.IsEmpty()) {        
        AfxMessageBox(_T("The Email Address cannot be empty"));
        return;
    } 
    */
    if (s1.IsEmpty()) {
        s1 = "";
    }
    if (s2.IsEmpty()) {
        s2 = DEFAULT_USER;
    }
    accountName = new wchar_t[ACCOUNT_PARAM_SIZE];
    replyAddress = new wchar_t[ACCOUNT_PARAM_SIZE];

    wcscpy(accountName, s1.GetBuffer( s1.GetLength() ));
    if(s1 != dispName)
        bSaveAccount = true;

    wcscpy( replyAddress, s2.GetBuffer( s2.GetLength() ));
    if(s2 != dispEmail)
        bSaveAccount = true;

    if( (accountName) && (replyAddress) && bSaveAccount) {
        SetCursor(LoadCursor(NULL, IDC_WAIT));
        HRESULT ret = -1;    
        ret = modifyAccount(accountName, replyAddress);
        SetCursor(LoadCursor(NULL, IDC_ARROW));
    }
    if(!checkStartSync()){
	    regsettings.writeToRegistry("mail", false);
    }

    if(accountName)
        delete [] accountName;
    if(replyAddress)
        delete [] replyAddress;

	CDialog::OnOK();	
}

void CMailSettings::OnMenuFolders()
{
	CMailFoldersSettings wndSettFolders;
    #ifdef WIN32_PLATFORM_WFSP
	    //show commandbar with ok/cancel
	    dlgCommandBar3.Show(false);
    #endif

	INT_PTR result = wndSettFolders.DoModal();

    #ifdef WIN32_PLATFORM_WFSP
	    dlgCommandBar3.Show(true);
    #endif
}

void CMailSettings::OnMenuFilters()
{
	CMailFiltersSettings wndSettFilters;

    #ifdef WIN32_PLATFORM_WFSP
	    //show commandbar with ok/cancel
	    dlgCommandBar3.Show(false);
    #endif

	INT_PTR result = wndSettFilters.DoModal();

    #ifdef WIN32_PLATFORM_WFSP
	    dlgCommandBar3.Show(true);	
    #endif
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CMailSettings::OnHotkey(WPARAM wParam, LPARAM lParam)
{
	if (HIWORD(lParam) == VK_TBACK)
		SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
	return 0;
}
#endif

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CMailSettings::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
    //int displayMode = DRA::GetDisplayMode();
    int resId;
    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
        resId = IDD_UI_MAIL_WIDE; break;
    case DRA::Portrait:
        resId = IDD_UI_MAIL; break;
    case DRA::Square:
        resId = IDD_UI_MAIL_SQUARE; break;
    default:
        resId = IDD_UI_MAIL; break;
    };

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));

    if( (resId == IDD_UI_MAIL_WIDE) || ( resId == IDD_UI_MAIL_SQUARE) )
        staticAdvanced.ShowWindow(FALSE);
    else
        staticAdvanced.ShowWindow(TRUE);        
}
#endif
