/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// RecoverSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "RecoverSettings.h"
#include "pim/utils.h"
#include "pim/Maincpp.h"
#include "base/startcmd.h"
#include "pim/SettingFunctions.h"
#include "RegistrySettings.h"
#include "pim/account.h"
#include "ClientUtil.h"

#include "winreg.h"

// CRecoverSettings dialog
void writeSourceForSlow(const wchar_t*  value);
IMPLEMENT_DYNAMIC(CRecoverSettings, CDialog)

CRecoverSettings::CRecoverSettings(CWnd* pParent /*=NULL*/)
    : CDialog(CRecoverSettings::IDD, pParent)
{

}

CRecoverSettings::~CRecoverSettings()
{
}

void CRecoverSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_RECOVER_CONTACTS, checkContacts);
    DDX_Control(pDX, IDC_RECOVER_CALENDAR, checkCalendar);
    DDX_Control(pDX, IDC_RECOVER_TASKS, checkTasks);
    DDX_Control(pDX, IDC_RECOVER_MAIL, checkMail);
    DDX_Control(pDX, IDC_RECOVER_BRIEFCASE, checkBriefcase);
    #if defined(WIN32_PLATFORM_PSPC)
        DDX_Control(pDX, IDC_RECOVER_NOTES, checkNotes);
    #endif
}


BEGIN_MESSAGE_MAP(CRecoverSettings, CDialog)
    ON_BN_CLICKED(IDC_RECOVER_MAIL, &CRecoverSettings::OnBnClickedRecoverMail)
    ON_BN_CLICKED(IDC_RECOVER_CALENDAR, &CRecoverSettings::OnBnClickedRecoverCalendar)
    ON_BN_CLICKED(IDC_RECOVER_CONTACTS, &CRecoverSettings::OnBnClickedRecoverContacts)
    ON_BN_CLICKED(IDC_RECOVER_TASKS, &CRecoverSettings::OnBnClickedRecoverTasks)
    ON_BN_CLICKED(IDC_RECOVER_BRIEFCASE, &CRecoverSettings::OnBnClickedRecoverBriefcase)
    #if defined(WIN32_PLATFORM_PSPC)
        ON_BN_CLICKED(IDC_RECOVER_NOTES, &CRecoverSettings::OnBnClickedRecoverNotes)
    #endif
END_MESSAGE_MAP()


// CRecoverSettings message handlers

BOOL CRecoverSettings::OnInitDialog()
{
    CString s1, s2;
    s1.LoadString(IDS_RECOVER); SetWindowText(s1); 
    CDialog::OnInitDialog();

    #if defined(WIN32_PLATFORM_WFSP)
        if (!dlgCommandBar3.Create(this) ||
            !dlgCommandBar3.InsertMenuBar(IDR_MENUBAR_RECOVER)){
                return FALSE;      // failed to create
        } 
        dlgCommandBar3.Show(true);
    #else
        ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
        dlgCommandBar.cbSize = sizeof(SHMENUBARINFO); 
        dlgCommandBar.hwndParent = this->GetSafeHwnd(); 
        dlgCommandBar.dwFlags = SHCMBF_HMENU;
        dlgCommandBar.nToolBarId = IDM_RECOVER; 
        dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
        dlgCommandBar.nBmpId = 0; 
        dlgCommandBar.cBmpImages = 0; 
        if(!SHCreateMenuBar(&dlgCommandBar)){
            AfxMessageBox(_T("Cannot create command bar!")); 
            return FALSE;
        }
    #endif
    //-- load string resources
    // TODO: !problem: does not load IDS_RECOVER_ITEMS string resource
    s2.LoadString(IDS_RECOVER_ITEMS);   SetDlgItemText(IDC_RECOVER_STATIC, s2);

   s1.LoadString(IDS_RECOVER_CONTACTS);	SetDlgItemText(IDC_RECOVER_CONTACTS, s1);
   s1.LoadString(IDS_RECOVER_CALENDAR);	SetDlgItemText(IDC_RECOVER_CALENDAR, s1);
   s1.LoadString(IDS_RECOVER_TASKS);   	SetDlgItemText(IDC_RECOVER_TASKS, s1);
   s1.LoadString(IDS_RECOVER_MAIL);    	SetDlgItemText(IDC_RECOVER_MAIL, s1);
   s1.LoadString(IDS_RECOVER_ITEMS);   	SetDlgItemText(IDC_RECOVER_STATIC, s1);
   s1.LoadString(IDS_RECOVER_BRIEFCASE);   SetDlgItemText(IDC_RECOVER_BRIEFCASE, s1);
        
    //-- check what sources are set to synctype none, and disable the checkboxes
    SyncRegSettings regsetting("all");

    if(!strcmp(regsetting.cs.getConfigSourcesParameter("contact", "sync"),"none"))
        checkContacts.EnableWindow(FALSE);

    if(!strcmp(regsetting.cs.getConfigSourcesParameter("calendar", "sync"),"none"))
        checkCalendar.EnableWindow(FALSE);
  
    if(!strcmp(regsetting.cs.getConfigSourcesParameter("task", "sync"),"none"))
        checkTasks.EnableWindow(FALSE);

    if(!strcmp(regsetting.cs.getConfigSourcesParameter("briefcase", "sync"),"none"))
        checkBriefcase.EnableWindow(FALSE);

    #if defined(WIN32_PLATFORM_PSPC)
    if(!strcmp(regsetting.cs.getConfigSourcesParameter("note", "sync"),"none"))
        checkNotes.EnableWindow(FALSE);
    #endif

    if(!strcmp(regsetting.cs.getConfigSourcesParameter("mail", "sync"),"none") ||
        !doesFunambolAccountExist())

        checkMail.EnableWindow(FALSE);
        EnableRecoverButton(false);
        checkMail.SetFocus();
    return FALSE;
}

void CRecoverSettings::OnOK()
{
    CString s1, s2;
    int nCheckedSources = 0;
    //-- prepare command line to provide to startsync
    SyncRegSettings regsetting("all", false);
    s2 = "";

    CString command("once");

    char chComma=',', chSpace=' ',chSeparator;
    BOOL isFirst = TRUE;
    
    wchar_t errorMsg[128];
    memset(errorMsg, 0, 128*sizeof(wchar_t));

    if (checkContacts.GetCheck() == BST_CHECKED) {

        chSeparator = (isFirst) ? chSpace : chComma;
        if(isFirst) 
            isFirst=FALSE;
        command = command + chSeparator + CONTACTS_NAME;
        regsetting.cs.setConfigSourcesParameter("contact", "last", "1");
        regsetting.cs.setConfigSourcesParameter("contact", "sync", "two-way");    
        nCheckedSources ++;
    }

    if(checkCalendar.GetCheck() == BST_CHECKED) {
        chSeparator = (isFirst) ? chSpace : chComma;
        if(isFirst) 
            isFirst=FALSE;
        command = command + chSeparator + APPOINTMENTS_NAME;
        regsetting.cs.setConfigSourcesParameter("calendar", "last", "1");
        regsetting.cs.setConfigSourcesParameter("calendar", "sync", "two-way");
        nCheckedSources++;
    }
    if(checkTasks.GetCheck() == BST_CHECKED) {
        chSeparator = (isFirst) ? chSpace : chComma;
        if(isFirst) 
            isFirst=FALSE;
        command = command + chSeparator + TASKS_NAME;
        regsetting.cs.setConfigSourcesParameter("task", "last", "1");        
        regsetting.cs.setConfigSourcesParameter("task", "sync", "two-way");
        nCheckedSources++;
    }
    if(checkBriefcase.GetCheck() == BST_CHECKED) {
        chSeparator = (isFirst) ? chSpace : chComma;
        if(isFirst) 
            isFirst=FALSE;
        command = command + chSeparator + FILES_NAME;
        regsetting.cs.setConfigSourcesParameter("briefcase", "last", "1");
        regsetting.cs.setConfigSourcesParameter("briefcase", "sync", "two-way");
        nCheckedSources++;
    }
    if(checkMail.GetCheck() == BST_CHECKED && doesFunambolAccountExist()) {
     chSeparator = (isFirst) ? chSpace : chComma;
     if(isFirst) 
         isFirst=FALSE;
     command = command + chSeparator + MAILS_NAME;
     regsetting.cs.setConfigSourcesParameter("mail", "last", "1");
     regsetting.cs.setConfigSourcesParameter("mail", "sync", "two-way");
     nCheckedSources++;   
    }
    #if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED) {
         chSeparator = (isFirst) ? chSpace : chComma;
         if(isFirst) 
             isFirst=FALSE;
         command = command + chSeparator + NOTES_NAME;
         regsetting.cs.setConfigSourcesParameter("note", "last", "1");
         regsetting.cs.setConfigSourcesParameter("note", "sync", "two-way");
         nCheckedSources++;   
        }
    #endif

    if(nCheckedSources != 0){
        regsetting.writeToRegistry("all", true);
        writeSourceForSlow(command);
        //CString cline("once");
        LPTSTR lpszCommand = command.GetBuffer(command.GetLength());

        //-- start slow-sync process
        int check  = checkStartSync();
        if(check == 0) { 

            //wchar_t err[512]; wchar_t* path = TEXT("\\Storage\\Program Files\\Funambol"); int ret = synchronize(err, path, NULL);
            unsigned long pid = startProgram(SYNCAPP, lpszCommand);
            if (pid) {
                waitProcess(pid, 1000);   //-- wait 1 sec     
            } else {
                s1.LoadString(IDS_TEXT_STARTSYNC_NOT_FOUND);
                AfxMessageBox(s1);
            }
        }
    }
    
  command.ReleaseBuffer();
  EndDialog(1);
}

/*
 * Sets a property value.
 * @param value - the property value (zero terminated string)
*/
void writeSourceForSlow(const wchar_t*  value) {
    if ((value == NULL)) {
        return;
    }

    HKEY key;
    DWORD res;
    wchar_t *p = TEXT("sourcesForSlow");;    
    
    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            TEXT("\\Software\\Funambol\\SyncclientPIM"),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );
       

    RegSetValueEx(
            key,
            p,
            NULL,
            REG_SZ,  // we currently support only strings
            (UCHAR*)value,
            (wcslen(value)+1)*sizeof(WCHAR)
            );
        
    if (key != 0) {
        RegCloseKey(key);
    }
} 

#if defined(WIN32_PLATFORM_PSPC)
void CRecoverSettings::OnBnClickedRecoverNotes()
{
    EnableRecoverButton(isCheckboxChecked());
}
#endif


void CRecoverSettings::EnableRecoverButton(bool bEnable){
    HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());
    TBBUTTONINFO tbi;
    if (!hwndMB) 
        return;

    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    ::SendMessage (hwndMB, TB_GETBUTTONINFO, IDOK, (LPARAM)&tbi);

    if(bEnable)
        tbi.fsState = TBSTATE_ENABLED;
    else
        tbi.fsState &= ~TBSTATE_ENABLED;

    ::SendMessage (hwndMB, TB_SETBUTTONINFO, IDOK, (LPARAM)&tbi);
}
void CRecoverSettings::OnBnClickedRecoverMail()
{
    EnableRecoverButton(isCheckboxChecked());
}

void CRecoverSettings::OnBnClickedRecoverCalendar()
{
    EnableRecoverButton(isCheckboxChecked());
}

void CRecoverSettings::OnBnClickedRecoverContacts()
{
    EnableRecoverButton(isCheckboxChecked());
}

void CRecoverSettings::OnBnClickedRecoverTasks()
{
    EnableRecoverButton(isCheckboxChecked());
}

void CRecoverSettings::OnBnClickedRecoverBriefcase()
{
    EnableRecoverButton(isCheckboxChecked());
}

bool CRecoverSettings::isCheckboxChecked()
{
    bool value = false;
    value = (checkContacts.GetCheck() == BST_CHECKED) || 
            (checkCalendar.GetCheck() == BST_CHECKED) ||
            (checkTasks.GetCheck() == BST_CHECKED)    ||
            (checkBriefcase.GetCheck() == BST_CHECKED) ||
            (checkMail.GetCheck() == BST_CHECKED);

    #if defined(WIN32_PLATFORM_PSPC)
        value |= (checkNotes.GetCheck() == BST_CHECKED);
    #endif

    return value;
}