/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "Winbase.h"
#include <windows.h>
#include <oleauto.h>
#include "initguid.h"
#include "pimstore.h"
#include <stdio.h>

#include "base/startcmd.h"
#include "pim/maincpp.h"
#include "pim/Utils.h"
#include "pim/SettingFunctions.h"
#include "client/SyncClient.h"
#include "pim/ContactSyncSource.h"
#include "pim/CalendarSyncSource.h"
#include "pim/NoteSyncSource.h"
#include "pim/TaskSyncSource.h"
#include "pim/BriefcaseSyncSource.h"

#include "pim/mail/MailSyncSource.h"
#include "client/MailSourceManagementNode.h"
#include "client/DMTClientConfig.h"
#include "filter/SourceFilter.h"
#include "filter/ClauseUtil.h"

#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "syncml/formatter/Formatter.h"
#include "spds/spdsutils.h"
#include "Notify.h"
#include "notify/timed_msgbox.h"
#include "pim/account.h"
#include "OutOfMemoryException.h"

#include "pim/difftime.h"

#include "events/SyncItemListenerClient.h"
#include "events/SyncListenerClient.h"
#include "events/SyncSourceListenerClient.h"
#include "events/SyncStatusListenerClient.h"
#include "events/TransportListenerClient.h"
#include "event/SetListener.h"

#include "HwndFunctions.h"
#include "SyncResults.h"

#include "pim/CustomSyncClient.h"
#include <new.h>
#include <algorithm>
//
// USE the pim/ClientSettings.h. THis must be shared with the same ClientSettings of the 
// FunambolClient
//
#include "pim/ClientSettings.h" 


int checkRefreshFromServer(int type);
#define OL_MAIL 610
SyncSourceListenerClient* syncSourceListener ;

// it could be a string. it is a vector because better should be divide the sources
// and not get a string
vector<wstring> sourceNameForSlow;

wchar_t* getSourcesForSlow();
void resetSourceForSlow();
wstring getMailUnsent(MailSyncSource mailSource);
wchar_t* completeMailId(const wchar_t* path, const wchar_t* partialID);
wchar_t* findMailIdInFile(const wchar_t* fileName, const wchar_t* partialID);

TCHAR* getMailMaxMsgSize();

/*      
* Function to handle the memory failure. The size means the size of the memory required
* the -1 means the error code
*/
int operatorNewHandler(size_t size)
{                          
    throw OutOfMemoryException(size, -1);
}

/*      
 * Entry point from VB application. 
 * Accept and manage Variant array of String, long and int. 
 * The entry array is a String array in which elements rapresent SyncSource name.
 * Now accepted and managed element names are "contacts" and "appointments"
 * Array contains at least one element
 * @param vbErrorMsg   : the variable to store the message to return to vb GUI interface
 * @param path       : the install path of the SyncClient PIM application on the device
 */

DWORD WINAPI synchronization (wchar_t* vbErrorMsg, wchar_t* path) {
    
    char CONTACTS_A    [15];
    char APPOINTMENTS_A[15];       
    char TASKS_A       [15];
    char FAVORITES_A   [15];
    char FILES_A       [15];
    char NOTES_A       [15];
    char MAILS_A       [15];    
    
    wchar_t CONTACTS    [15];
    wchar_t APPOINTMENTS[15];       
    wchar_t TASKS       [15];
    wchar_t FAVORITES   [15];
    wchar_t FILES       [15];
    wchar_t NOTES       [15];
    wchar_t MAILS       [15];    
    
    wchar_t errorMessage[1024];
    int rc          = 0;
    lastErrorCode   = 0;
    ArrayList syncSources;
    char* value;
    errorMessage[0] = 0;
    SyncSource** ssArray = NULL;
    int ret = 0;
    BOOL isMailToSync = FALSE;
    int mailSourceIndex = -1; 
    int result = 0;
    strcpy(CONTACTS_A,      CONTACTS_NAME_A    );
    strcpy(APPOINTMENTS_A,  APPOINTMENTS_NAME_A);
    strcpy(TASKS_A,         TASKS_NAME_A       );
    strcpy(FAVORITES_A,     FAVORITES_NAME_A   );
    strcpy(FILES_A,         FILES_NAME_A       );
    strcpy(NOTES_A,         NOTES_NAME_A       );
    strcpy(MAILS_A,         MAILS_NAME_A       );
    
    wcscpy(CONTACTS,      CONTACTS_NAME    );
    wcscpy(APPOINTMENTS,  APPOINTMENTS_NAME);
    wcscpy(TASKS,         TASKS_NAME       );
    wcscpy(FAVORITES,     FAVORITES_NAME   );
    wcscpy(FILES,         FILES_NAME       );
    wcscpy(NOTES,         NOTES_NAME       );
    wcscpy(MAILS,         MAILS_NAME       );
    
    LogLevel ll = getLOGLevel();
    LOG.setLevel(ll);        
        
    int days = 0;
    int attachSize = 0;
    wchar_t t[12];
    TCHAR* msxmessagesize = getMailMaxMsgSize();

    wchar_t* downloadAge = NULL;
    SourceFilter* filter = NULL;
    DMTClientConfig config(APPLICATION_URI); 
    config.read();
    //SyncSourceConfig *sc;
  
    // 
    // The list of the sync sources to sync
    //
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);        
    ManagementNode* node = NULL;
    BOOL blockedByUser = FALSE;
    /*
    * To understand if the briefcase and note directory exist. If no and there are 
    * no other source to sync the message is specific
    */
    BOOL briefcaseDirExists = TRUE;
    BOOL noteDirExists = TRUE;


    // contact
    node = dmt->getManagementNode(CONTACTS_CONTEXT);
    value = node->readPropertyValue("sync");
    if (strcmp(value, "none") != 0) {
        
        //config.getSyncSourceConfig(CONTACTS_A);
        ContactSyncSource contactSource = ContactSyncSource(CONTACTS, OL_CONTACTS, config.getSyncSourceConfig(CONTACTS_A));             
        contactSource.setPath(path);
        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_CONTACTS);
        }        
        if (result == 0) {
            syncSources.add(contactSource);
            
            if(!strcmp(value,"slow"))
                SyncResultsLogger.syncTypes |= SYNCRESULTS_CONTACTS_SLOW;
        } else {
            blockedByUser = TRUE;
        }

    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
    
    // calendar
    node = dmt->getManagementNode(APPOINTMENTS_CONTEXT);    
    value = node->readPropertyValue("sync");
    if (strcmp(value, "none") != 0) {

        //config.getSyncSourceConfig(APPOINTMENTS_A, sc);
        CalendarSyncSource  s  = CalendarSyncSource (APPOINTMENTS, OL_CALENDAR, config.getSyncSourceConfig(APPOINTMENTS_A));         
        s.setPath(path);
        
        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_CALENDAR);
        }        
        if (result == 0) {           
            syncSources.add(s);
           
            if(!strcmp(value,"slow"))
                SyncResultsLogger.syncTypes |= SYNCRESULTS_CALENDAR_SLOW;
        } else {
            blockedByUser = TRUE;
        }
    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
    
    // task
    node = dmt->getManagementNode(TASKS_CONTEXT);
    value = node->readPropertyValue("sync");
    if (strcmp(value, "none") != 0) {
        
        //config.getSyncSourceConfig(TASKS_A, sc);
        TaskSyncSource  s  = TaskSyncSource (TASKS, OL_TASK, config.getSyncSourceConfig(TASKS_A));         
        s.setPath(path);

        result = 0;
        if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
            result = checkRefreshFromServer(OL_TASK);
        }        
        if (result == 0) {           
            syncSources.add(s);        

            if(!strcmp(value,"slow"))
                SyncResultsLogger.syncTypes |= SYNCRESULTS_TASKS_SLOW;
        } else {
            blockedByUser = TRUE;
        }
    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
    
    // briefcase
    node = dmt->getManagementNode(BRIEFCASE_CONTEXT);
    value = node->readPropertyValue("sync");
    if (strcmp(value, "none") != 0) {

        //config.getSyncSourceConfig(FILES_A, sc);
        BriefcaseSyncSource  s  = BriefcaseSyncSource (FILES, OL_BRIEFCASE, config.getSyncSourceConfig(FILES_A));         
        s.setPath(path);
        
        char* d = node->readPropertyValue("dir");
        wchar_t *dir = toWideChar(d);
        
        if (existsDirectory(dir)) {
            
            s.setDir(dir);        
            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_BRIEFCASE);
            }        
            if (result == 0) {      
                syncSources.add(s);       

                if(!strcmp(value,"slow"))
                    SyncResultsLogger.syncTypes |= SYNCRESULTS_BRIEFCASE_SLOW;
                } else {
                    blockedByUser = TRUE;
            }
        } else {
            LOG.info("Briefcase dir %S doesn't exists!", dir);
            briefcaseDirExists = FALSE;

        }
        if (dir) { delete [] dir; dir = NULL; }
        if (d)   { delete [] d;}
    }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }

     // note
    node = dmt->getManagementNode(NOTES_CONTEXT);
    value = node->readPropertyValue("sync");
    if (strcmp(value, "none") != 0) {

        //config.getSyncSourceConfig(NOTES_A, sc);
        NoteSyncSource  s  = NoteSyncSource (NOTES, OL_NOTES, config.getSyncSourceConfig(NOTES_A));         
        s.setPath(path);

        char* d = node->readPropertyValue("dir");
        wchar_t *dir = toWideChar(d);

        if (existsDirectory(dir)) {
            s.setDir(dir);
            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_NOTES);
            }        
            if (result == 0) {          
                syncSources.add(s);      

            } else {
                blockedByUser = TRUE;
            }
        } else {
            LOG.info("Note dir %S doesn't exists!", dir);
            noteDirExists = FALSE;
        }
        if (dir) { delete [] dir; dir = NULL; }
        if (d)   { delete [] d;}
    }
    if (dmt) { delete dmt; dmt = NULL; }
    if (node) { delete node ; node = NULL; }
    if (value) { delete [] value; value = NULL; }
    
    /*
    * Try to insert MAIL as a source to sync with the other to try the Large Object support
    */
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");   
    MailSyncSourceConfig sc;    
    sc = m.getMailSourceConfig(true);    
    MailSyncSource mailSource = MailSyncSource (MAILS, &sc); 
    if (strcmp(sc.getSync(), "none") != 0 && doesFunambolAccountExist()) {
                 
        result = 0;
        if (strcmp(sc.getSync(), "refresh-server") == 0 || strcmp(sc.getSync(), "refresh") == 0) {                    
            result = checkRefreshFromServer(OL_MAIL);                       
        }      
        if (result == 0) {          
                      
            getFolderToSync(sc, t);    
        
            days = sc.getDownloadAge();
            downloadAge = getDayBefore(days);
            attachSize = sc.getAttachSize();

            if (attachSize == -1) {
            //reserved value for the remain part of syncml msg
            // 1024 because the filter multiply for 1024
                attachSize = (wcstol(msxmessagesize, NULL, 10) - 32768) / 1024;  
            }

            filter = 
                ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(), attachSize);        
            
            //
            //
            // create filter as default

            mailSource.setPath(path);
            mailSource.setFolderToSync(t);
            mailSource.setFilter(filter);
            
            syncSources.add(mailSource);    

            if(!strcmp(sc.getSync(),"slow")) {
                SyncResultsLogger.syncTypes |= SYNCRESULTS_MAIL_SYNC;
            }
        } else {
            blockedByUser = TRUE;
        }

    }       

    int sourcesNumber = syncSources.size();

    /*
    //------- find if mail is to be synchronized
    MailSourceManagementNode m1 = MailSourceManagementNode(SOURCES_CONTEXT, "mails");   
    //MailSyncSourceConfig sc1;    
    MailSyncSourceConfig sc = m1.getMailSourceConfig(true);    
    //MailSyncSource mailSource = MailSyncSource (MAILS, &sc1);    
    
    if (strcmp(sc.getSync(), "none") != 0) 
    {
        sourcesNumber += 1;
        SyncResultsLogger.flagMail |= SYNCRESULTS_MAIL_SYNC;  //-- mail is to be synced
    }
    */
    
    if (syncSources.size() == 0) {
            LOG.debug("The source array to sync is empty");
        if (blockedByUser) {
            wsprintf(errorMessage, TEXT("No sources to sync: blockedByUser"));
            LOG.debug("%s", ("No sources to sync: blockedByUser"));
            ret = -20;  
        } else if (briefcaseDirExists == FALSE) {
            wsprintf(errorMessage, TEXT("No sources to sync. Briefcase folder doesn't exists"));
            LOG.debug("%s", ("No sources to sync: briefcaseDirExists false"));
            ret = -25;  
        } else if (noteDirExists == FALSE) {
            wsprintf(errorMessage, TEXT("No sources to sync. Note folder doesn't exists"));
            LOG.debug("%s", ("No sources to sync: noteDirExists false"));
            ret = -26;  
        } else {
            wsprintf(errorMessage, TEXT("No sources to sync"));
            LOG.debug("%s", ("No sources to sync"));
            ret = -10;  

            // TODO: send SOURCES SIZE to UI (if present)
            if(HwndFunctions::wnd)
                SendMessage(HwndFunctions::wnd, (WM_APP+5), (WPARAM) SyncResultsLogger.flagMail,
                (LPARAM) sourcesNumber);
            SyncResultsLogger.totalSources = sourcesNumber;

        }
            
    }
/*
    if (syncSources.size() == 0 && blockedByUser) {
        wsprintf(errorMessage, TEXT("No sources to sync: blockedByUser"));
        LOG.debug("%s", ("No PIM sources to sync: blockedByUser"));
        ret = -20;  

        // TODO: send SOURCES SIZE to UI (if present)
        
    } else if (syncSources.size() == 0) {
        wsprintf(errorMessage, TEXT("No sources to sync"));
        LOG.debug("%s", ("No PIM sources to sync"));
        ret = -10;  

        // TODO: send SOURCES SIZE to UI (if present)
        if(HwndFunctions::wnd)
            SendMessage(HwndFunctions::wnd, (WM_APP+5), (WPARAM) SyncResultsLogger.flagMail,
            (LPARAM) sourcesNumber);
        SyncResultsLogger.totalSources = sourcesNumber;
    
    } 
*/
    else {
        
        // SyncClient syncClient(config);
        // SyncClient syncClient;        
                
        CustomSyncClient syncClient;   
        
        wchar_t* ssfs = getSourcesForSlow();
        sourceNameForSlow.push_back(ssfs);
        syncClient.namesOfSources = sourceNameForSlow;
        syncClient.isMailInclusive = FALSE;  // here it is not the inclusive filter for sure
        syncSourceListener->sc = &syncClient;
        delete [] ssfs;
        resetSourceForSlow();
        
        int l = syncSources.size();

        // TODO: send SOURCES SIZE to UI (if present)
        if(HwndFunctions::wnd)
            SendMessage(HwndFunctions::wnd, (WM_APP+5), (WPARAM) SyncResultsLogger.flagMail,
            (LPARAM) sourcesNumber);
        SyncResultsLogger.totalSources = sourcesNumber;
        //sourcesNumber += l;

        const int ml = 10;        
        SyncSource* array[ml];    
        for (int i=0; (i<l) && (i<ml-1); ++i) {
            if (wcscmp(((SyncSource*)syncSources[i])->getName(), TEXT("mail")) == 0) {
                array[i] = &mailSource;
            } else {
                array[i] = (SyncSource*)syncSources[i];
            }
        }
        
        array[l] = NULL;
        ret = syncClient.sync(config, array);
        
        // save all configurations at time if there are no errors
        if (ret == 0)
            config.save();
        else if (ret > 299 && ret < 599) {
            // try to discover if there are sources that are ok...
            SyncSource* s = NULL;
            for (int i = 0; (s = array[i]) != NULL; i++) {
                SyncSourceReport* sr = s->getReport();
                if (sr->checkState()) {
                    SyncSourceConfig sc = s->getConfig();
                    ClientSettings cs(APPLICATION_URI);                    
                    cs.saveSyncSourceConfig(sc);                    
                }
            }
        }

        if (mailSource.getIsMailInOutbox() && ret == 0) {
             LOG.debug("synchronization: There are mails in outbox");           
        }

        //
        // Check if the mail has to be synched 2 times...
        //
/*
        if (mailSource.getIsMailInOutbox() && ret == 0) {
            
            SyncSource** ssArray = new SyncSource*[2];

            ssArray[0] = &mailSource;
            ssArray[1] = NULL;

            sc = m.getMailSourceConfig(true); // refresh the config of the mailSyncSource

            ret = syncClient.sync(config, ssArray);

            if (ret == 0) {
                
                config.save();            
            }
        }
 */       
        if (mailSource.getIsMailInInbox()) {
            wchar_t s[256];
            wcscpy(s, path);
            wcscat(s, TEXT("\\newmessage.wav"));
            if (!PlaySound(s, NULL, SND_SYNC | SND_NODEFAULT))
                MessageBeep(MB_ICONASTERISK);
        }
        
        if (mailSource.getFailedSendMailInOutbox()) {
            //TimedMessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
            //    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, OUT_MESSAGE_TIMEOUT*1000);
            wstring message = getMailUnsent(mailSource);
            MessageBox(NULL, message.c_str(), TEXT("Alert"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            /*
            if (getNumberMailUnsent(mailSource) > 1) {
                
                MessageBox(NULL, MORE_OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            }
            else 
                MessageBox(NULL, ONE_OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            */
        }
        


    }
    
    //
    // NOTE: when passing a string from VB to a DLL, it looks like
    // VB does not want that the last chararcter in the string is
    // the terminator. Therefore we change it to a space. However,
    // IT IS VERY IMPORTANT that the string allocated by the caller
    // less than the length of errorMessage. 
    //
    wsprintf(vbErrorMsg, errorMessage);
    vbErrorMsg[wcslen(vbErrorMsg)] = L' ';
    vbErrorMsg[wcslen(vbErrorMsg)+1] = 0;    
    LOG.info("Sync ended.");
    
    return ret;

}

/*      
 * Entry point to kickoff the synchronization process, specifying which
 * sources to synchronize.
 *
 * @param path    : the install path of the SyncClient PIM application on 
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 * @param ids :     NULL terminated array of strings containing the id of the mail
 *                  that must be synched with a INCLUSIVE filter. If ids is NULL, the incluse filter
 *                  is not used. Maybe if neede is created the exclusive one
 *
 */

DWORD WINAPI synchronizeMultipleSources (const wchar_t* path, const wchar_t** sources, const wchar_t** ids) {
    
    if (sources == 0) {
        return 0;
    }
    
    int sourcesNumber = 0, ret = 0, i = 0;
    int mailSourceIndex = -1, result = 0;        
    int days = 0, attachSize = 0;
    BOOL blockedByUser = FALSE;

    TCHAR* msxmessagesize = getMailMaxMsgSize();
    wchar_t* downloadAge = NULL;
    SourceFilter* filter = NULL;
    
    ArrayList syncSources;
    char* value = NULL;
    
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);    
    ManagementNode* node = NULL;
        
    LOG.setLevel(getLOGLevel());    
    
    // used to store the previous mail sync mode. It needs to be used in the 
    // inclusive mothod. The previous is stored and it is substituted with the one-way-from-server
    // then after the sync is finished, the old sync mode is restored
    string previousMailSyncMode = "";

    DMTClientConfig config(APPLICATION_URI); 
    config.read();    
    
    //
    // Mail configuration read now because it has to have a scope until the end
    //
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");   
    MailSyncSourceConfig sc;    
    sc = m.getMailSourceConfig(true);    
    MailSyncSource mailSource = MailSyncSource (TEXT("mail"), &sc); 


    for (i=0; sources[i]; i++) {
        if (wcscmp(sources[i], CONTACTS_NAME) == 0) {

            // config.getSyncSourceConfig(CONTACTS_NAME_A, sc);
            ContactSyncSource s(CONTACTS_NAME, OL_CONTACTS, config.getSyncSourceConfig(CONTACTS_NAME_A));
            s.setPath(path);

            node = dmt->getManagementNode(CONTACTS_CONTEXT);
            value = node->readPropertyValue("sync");
            
            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_CONTACTS);
            }        
            if (result == 0) {
                syncSources.add(s);
                
                if(!strcmp(value,"slow"))
                    SyncResultsLogger.syncTypes |= SYNCRESULTS_CONTACTS_SLOW;
            } else {
                blockedByUser = TRUE;
            }
            if (node) { delete node; node = NULL; }
            if (value) { delete [] value; value = NULL; }

        }
        else if (wcscmp(sources[i], APPOINTMENTS_NAME) == 0) {

            //config.getSyncSourceConfig(APPOINTMENTS_NAME_A, sc);
            CalendarSyncSource s(APPOINTMENTS_NAME, OL_CALENDAR, config.getSyncSourceConfig(APPOINTMENTS_NAME_A));
            s.setPath(path);
            
            node = dmt->getManagementNode(APPOINTMENTS_CONTEXT);
            value = node->readPropertyValue("sync");

            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_CALENDAR);
            }        
            if (result == 0) {
                syncSources.add(s);
                if(!strcmp(value,"slow"))
                    SyncResultsLogger.syncTypes |= SYNCRESULTS_CALENDAR_SLOW;
            } else {
                blockedByUser = TRUE;
            }

            if (node) { delete node; node = NULL; }
            if (value) { delete [] value; value = NULL; }
        }
        else if (wcscmp(sources[i], TASKS_NAME)        == 0) {

            // config.getSyncSourceConfig(TASKS_NAME_A, sc);
            TaskSyncSource  s(TASKS_NAME, OL_TASK, config.getSyncSourceConfig(TASKS_NAME_A));         
            s.setPath(path);
            
            node = dmt->getManagementNode(TASKS_CONTEXT);
            value = node->readPropertyValue("sync");

            result = 0;
            if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                result = checkRefreshFromServer(OL_TASK);
            }        
            if (result == 0) {
                syncSources.add(s);
                if(!strcmp(value,"slow"))
                    SyncResultsLogger.syncTypes |= SYNCRESULTS_TASKS_SLOW;
            } else {
                blockedByUser = TRUE;
            }

            if (node) { delete node; node = NULL; }
            if (value) { delete [] value; value = NULL; }
        }
        else if (wcscmp(sources[i], FAVORITES_NAME)    == 0) {
/*
            FavoritesSyncSource s(FAVORITES_NAME, OL_FAVORITES);
            s.setPath(path);
            syncSources.add(s);
*/
        }
        else if (wcscmp(sources[i], FILES_NAME)        == 0) {   

            // config.getSyncSourceConfig(FILES_NAME_A, sc);
            BriefcaseSyncSource  s(FILES_NAME, OL_BRIEFCASE, config.getSyncSourceConfig(FILES_NAME_A));       
            s.setPath((wchar_t*)path);
            
            node = dmt->getManagementNode(BRIEFCASE_CONTEXT);
            if (node) {
                char* d = node->readPropertyValue("dir");
                wchar_t* dir = toWideChar(d);
                if (existsDirectory(dir)) {
                s.setDir(dir);
                               
                value = node->readPropertyValue("sync");

                result = 0;
                if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                    result = checkRefreshFromServer(OL_BRIEFCASE);
                }        
                if (result == 0) {
                    syncSources.add(s);
                    if(!strcmp(value,"slow"))
                        SyncResultsLogger.syncTypes |= SYNCRESULTS_BRIEFCASE_SLOW;
                    } else {
                        blockedByUser = TRUE;
                    }
                }
                if (node) { delete node; node = NULL; }
                if (value) { delete [] value; value = NULL; }
                if (dir) { delete [] dir; dir = NULL; }
                if (d)   { delete [] d; d = NULL; }
            }
            else {
                LOG.error("Can't access management node %s", BRIEFCASE_CONTEXT);
            }
        } 
        else if (wcscmp(sources[i], NOTES_NAME) == 0) {

            // config.getSyncSourceConfig(NOTES_NAME_A, sc);
            NoteSyncSource s(NOTES_NAME, OL_NOTES, config.getSyncSourceConfig(NOTES_NAME_A));         
            s.setPath(path);
            
            node = dmt->getManagementNode(NOTES_CONTEXT);
            if (node) {
                char* d = node->readPropertyValue("dir");
                wchar_t* dir = toWideChar(d);
                s.setDir(dir);
                
                value = node->readPropertyValue("sync");

                result = 0;
                if (strcmp(value, "refresh-server") == 0 || strcmp(value, "refresh") == 0) {
                    result = checkRefreshFromServer(OL_NOTES);
                }        
                if (result == 0) {
                    syncSources.add(s);
                   
                } else {
                    blockedByUser = TRUE;
                }

                if (node) { delete node; node = NULL; }
                if (value) { delete [] value; value = NULL; }
                if (dir) { delete [] dir; dir = NULL; }
                if (d) { delete [] d; d = NULL; }
            } 
            else {
                LOG.error("Can't access management node %s", NOTES_CONTEXT);
            }
        }  
        else if (wcscmp(sources[i], MAILS_NAME) == 0 && doesFunambolAccountExist()) {
            
            // the config of the mail is read at the beginning of the loop
                                                                   
            result = 0;
            if (strcmp(sc.getSync(), "refresh-server") == 0 || strcmp(sc.getSync(), "refresh") == 0) {                    
                result = checkRefreshFromServer(OL_MAIL);                       
            }      
            if (result == 0) {          
                
                wchar_t t[12];            
                getFolderToSync(sc, t);    
                
                attachSize = sc.getAttachSize();
                
                // we want to disable the message size filter for the inclusive items
                // but we need to use the default as for donwload all message
                if (ids)
                    attachSize = -1;

                if (attachSize == -1) {
                    //reserved value for the remain part of syncml msg
                    // 1024 because the filter multiply for 1024
                    attachSize = (wcstol(msxmessagesize, NULL, 10) - 32768) / 1024;  
                }

                //
                // the ids is an array of mail id 
                //
                if (ids) {
                    //                    
                    mailSource.setSyncMode(SYNC_ONE_WAY_FROM_SERVER);
                    wchar_t* idComplete = completeMailId(path, ids[0]);
                    if (idComplete) {
                        char* ss = toMultibyte(idComplete);
                        filter = ClauseUtil::createSourceFilterInclusive(ss, attachSize);                
                        delete [] ss;
                        delete [] idComplete;
                    }
                } else {
                    days = sc.getDownloadAge();
                    downloadAge = getDayBefore(days);                

                    filter = 
                        ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(), attachSize);        
                }
                                
                mailSource.setPath(path);
                mailSource.setFolderToSync(t);

                mailSource.setFilter(filter);
                
                syncSources.add(mailSource);    
                if(!strcmp(sc.getSync(),"slow")) {
                    SyncResultsLogger.syncTypes |= SYNCRESULTS_MAIL_SYNC;
                }
            } else {
                blockedByUser = TRUE;
            }                                 
        }
    }
       
    if (syncSources.size() == 0 && blockedByUser) {
        LOG.debug("%s", ("No PIM sources to sync: blockedByUser"));
        ret = -20;
      
    
    } 
    if (syncSources.size() == 0) {
       LOG.debug("%s", ("No PIM sources to sync"));
       ret = -10;
       //sourcesNumber = 0;
       if(HwndFunctions::wnd)
           SendMessage(HwndFunctions::wnd, (WM_APP+5), (WPARAM) SyncResultsLogger.flagMail,
           (LPARAM) sourcesNumber); 
       SyncResultsLogger.totalSources = sourcesNumber;
    } 
     else {
        //
        // Now we can create the Sync4jClient object and kick off the sync
        //
        // SyncClient syncClient;
        CustomSyncClient syncClient;
        
        wchar_t* ssfs = getSourcesForSlow();
        sourceNameForSlow.push_back(ssfs);

        // which sources are to recover, used by SyncResults
        vector<wstring> recoverSources;
        wstring s1(ssfs); 
        int pos1 = s1.rfind(_T(" "));
        int pos2 = s1.rfind(_T(","));
        if( (pos1!=wstring::npos) && (pos2 == wstring::npos)){
            recoverSources.push_back(s1.substr(pos1+1, s1.size()));
        }
        else{
            s1.replace(0,pos1,_T(""));
            pos2 = s1.rfind(_T(","));
            while(pos2 != wstring::npos){
                recoverSources.push_back(s1.substr(pos2+1,s1.size()));
                s1.replace(pos2,s1.size(),_T(""));
                pos2 =  s1.rfind(_T(","));
            }
            if(s1.size() != 0)
                recoverSources.push_back(s1.substr(1,s1.size()));
        };

        for(unsigned int j=0;j<recoverSources.size();j++){
            if(wcscmp(recoverSources[j].c_str(),_T("mail")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_MAIL_SLOW;

            if(wcscmp(recoverSources[j].c_str(),_T("calendar")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_CALENDAR_SLOW;

            if(wcscmp(recoverSources[j].c_str(),_T("contact")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_CONTACTS_SLOW;

            if(wcscmp(recoverSources[j].c_str(),_T("task")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_TASKS_SLOW;

            if(wcscmp(recoverSources[j].c_str(),_T("briefcase")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_BRIEFCASE_SLOW;
        }

        
        syncClient.namesOfSources = sourceNameForSlow;
        if (ids)
            syncClient.isMailInclusive = TRUE;
        else
            syncClient.isMailInclusive = FALSE;

        syncSourceListener->sc = &syncClient;
        delete [] ssfs;
        resetSourceForSlow();
                
        int l = syncSources.size();
        sourcesNumber += l;

        // TODO: send SOURCES SIZE to UI (if present)
        if(HwndFunctions::wnd)
            SendMessage(HwndFunctions::wnd, (WM_APP+5), (WPARAM) SyncResultsLogger.flagMail,
            (LPARAM) sourcesNumber); // TODO: 
        SyncResultsLogger.totalSources = sourcesNumber;


        const int ml = 10;        
        SyncSource* s_array[ml];
        for (i=0; (i<l) && (i<ml-1); ++i) {
            if (wcscmp(((SyncSource*)syncSources[i])->getName(), TEXT("mail")) == 0) {
                s_array[i] = &mailSource;
            } else {
                s_array[i] = (SyncSource*)syncSources[i];
            }
        }       
        s_array[i] = NULL;
        ret = syncClient.sync(config, s_array);
                
        if (ret == 0)
            config.save();
        else if (ret > 299 && ret < 599) {
            // try to discover if there are sources that are ok...
            SyncSource* s = NULL;
            for (int i = 0; (s = s_array[i]) != NULL; i++) {
                SyncSourceReport* sr = s->getReport();
                if (sr->checkState()) {
                    SyncSourceConfig sc = s->getConfig();                    
                    ClientSettings cs(APPLICATION_URI);                    
                    cs.saveSyncSourceConfig(sc);                    
                }
            }
        }
        
        if (mailSource.getIsMailInOutbox() && ret == 0) {
             LOG.debug("synchronizeMultipleSources: There are mails in outbox");           
        }

        if (mailSource.getIsMailInInbox()) {
            wchar_t s[256];
            wcscpy(s, path);
            wcscat(s, TEXT("\\newmessage.wav"));
            if (!PlaySound(s, NULL, SND_SYNC | SND_NODEFAULT))
                MessageBeep(MB_ICONASTERISK);
        }
        
        if (mailSource.getFailedSendMailInOutbox()) {
            //TimedMessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
            //    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, OUT_MESSAGE_TIMEOUT*1000);
            wstring message = getMailUnsent(mailSource);
            MessageBox(NULL, message.c_str(), TEXT("Alert"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            
            /*
            if (getNumberMailUnsent(mailSource) > 1)
                MessageBox(NULL, MORE_OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            else 
                MessageBox(NULL, ONE_OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            //MessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
            //    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            */
        }

        LOG.debug("Sync ended multiple sources.");
    }
    return ret;
}
 
int startProgram(const wchar_t *app, const wchar_t *cmdline)
{
    // reset the sourceNameForSlow array
    sourceNameForSlow.clear();

    wchar_t path[256];  
    PROCESS_INFORMATION procinfo;
    ClientSettings cs(APPLICATION_URI);
    cs.readConfig();
    string s = cs.getPath(); wchar_t* path2 = toWideChar(s.c_str());
    wcscpy(path, path2); delete [] path2;

    wchar_t cmd[512];
    wsprintf(cmd, TEXT("%s\\%s"), path, app);

    LOG.debug("Running: %ls %ls\n", cmd, cmdline);
    if( CreateProcess( cmd, cmdline, 
        NULL, NULL, FALSE, 0,
        NULL, NULL, NULL, &procinfo ) ) {
                return procinfo.dwProcessId; 
    }
    else {
        //-- maybe the Registry entry is missing, trying startcmd()
        int pid = startcmd(app, cmdline);
        return pid;
    }
}

/**
 * Return TRUE if the process is started
 */
BOOL startPolling()
{    
    //return (startcmd(SYNCAPP, APP_RUN_AT_TIME) > 0);
    return (startProgram(SYNCAPP, APP_RUN_AT_TIME) > 0);
}

/*
 * Return 0: process terminated successfully
 *        >0: error code from child process    
 *       -1: no such process/invalid handle
 *       -2: timeout
 *       -3: can't get child exit code
 *       -5: can't run the app
 *       -6: the application is stopped by user (used into startSyncPid and checkPid)
 */

int startSync(time_t timeout)
{
    if (!timeout)
        timeout=5*60; // default timeout 5 minutes

    //unsigned long pid = startcmd(SYNCAPP, TEXT("once"));
    unsigned long pid = startProgram(SYNCAPP, TEXT("once"));
   
    if (pid) {
        return waitProcess(pid, timeout*1000);        
    }
    else {
        return -5;        
    }
    
}

/*
* Used from evb interface. It returns:
* . 1000 if the handle is NULL; so the processId is not valid or not assiciated to this process
* . STILL_ACTIVE (259) the process is still active.
* . a different value indicates the proper exit value.
*/

int checkPid(long pid) {

    unsigned long use = pid;
    DWORD exitCode = 1000;
    wchar_t t[128];
    
    wsprintf(t, TEXT("%lu"), use);    
    
    HANDLE phandle = OpenProcess(0, FALSE, use);  
    
    if (phandle) {
        BOOL f = GetExitCodeProcess(phandle, &exitCode);
        CloseHandle(phandle);        
    } 
    
    return exitCode;
    
}

unsigned long startSyncPid(time_t timeout)
{
    if (!timeout)
        timeout=5*60; // default timeout 5 minutes

    //unsigned long pid = startcmd(SYNCAPP, TEXT("once"));
    unsigned long pid = startProgram(SYNCAPP, TEXT("once"));
    
    if (pid) {
        return pid;
    }
    else {
        return -5;
    }
}


/*      
 * Entry point to kickoff the synchronization process, both specifying the 
 * sources or not. The default is to make all the configured sources.
 *
 * @param path    : the install path of the SyncClient PIM application on 
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 * @param ids :     NULL terminated array of strings containing the id of the mail
 *                  that must be synched with a INCLUSIVE filter. Note that this parameter
 *                  could be valorized only when there is only a source and this source is mail
 *
 * To avoid a server bug and send every time the capabilities to the server
 * in order to receive the NumberOfChanges tag
 * RESET the hashDevInfo flag in the registry
 */

DWORD WINAPI synchronize (wchar_t *errmsg, const wchar_t* path, const wchar_t** sources, const wchar_t** ids)
{
    wcscpy(errmsg, TEXT("Ok."));
    LOG.reset(); 
    int ret = 0;

    HwndFunctions::getWindowHandle();

    SyncItemListenerClient* itemListener = new SyncItemListenerClient();
    SyncListenerClient* syncListener = new SyncListenerClient();
    syncSourceListener  = new SyncSourceListenerClient();
    SyncStatusListenerClient* sstl = new SyncStatusListenerClient();
    TransportListenerClient* tl = new TransportListenerClient();
    
    setSyncItemListener(itemListener);
    setSyncListener(syncListener);
    setSyncSourceListener(syncSourceListener);
    setSyncStatusListener(sstl);
    setTransportListener(tl);
    
    SyncResultsLogger.reset();
    
    //
    // reset the devinfhash. This should be removed after server fix
    //
    ClientSettings cs(APPLICATION_URI);
    cs.readConfig(false);
    cs.getDeviceConfig().setDevInfHash("0");
    cs.saveConfig();
    
    if (sources)
        ret = synchronizeMultipleSources(path, sources, ids);
    else
        ret = synchronization(errmsg, (wchar_t *)path);
    
    if (ret == -20) {
        LOG.debug("The user stops the sync to avoid slow sync of PIM");
        return ret;
    }    
    
    if (ret == -10) {        
        LOG.debug("Mail not set to be synched");
        LOG.debug("No source to sync");
        return -10;
    }
    
    if (ret != -10) {
        SyncResultsLogger.writeMailSyncResults();
        SyncResultsLogger.writePimSyncResults();        
    }
       
    //-- release listeners
    if(itemListener)
    { delete itemListener; itemListener=NULL; }
    if(syncListener)
    { delete syncListener; syncListener=NULL; }
    if(syncSourceListener)
    { delete syncSourceListener; syncSourceListener=NULL; }
    if(itemListener)
    { delete itemListener; itemListener=NULL; }
    if(itemListener)
    { delete itemListener; itemListener=NULL; }

    if(ret != 0)
        return ret;    
    
    return 0;
}

/*
    LOG_LEVEL_NONE  = 0
    LOG_LEVEL_INFO  = 1
    LOG_LEVEL_DEBUG = 2
*/

LogLevel getLOGLevel() {
    
    ClientSettings cs(APPLICATION_URI);
    cs.readConfig();
    
    LogLevel ret = LOG_LEVEL_NONE;

    ret = (LogLevel)cs.getDeviceConfig().getLogLevel();
    return ret;

}


/*
 * NOTE: USED ONLY TEMPORANEALLY TO GET A MAIL PARAMETER THAT MUST
 * BE FIXED IN A PROPER WAY
 */
TCHAR* getMailMaxMsgSize() {
    HKEY key = NULL;    
    DWORD res;    
    long err = 0;
    TCHAR *p = TEXT("mailMaxMsgSize");
    ULONG dim = 0;
    TCHAR *buf = NULL;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            TEXT("\\Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        sprintf(lastErrorMsg, "Invalid context path: %s", "Software/Funambol/SyncclientPIM/spds/sources/mails");
        goto finally;
    }

    // Get value length
    err = RegQueryValueEx(
            key,
            p,
            NULL,
            NULL,  // we currently support only strings
            NULL,
            &dim
            );

    if (err == ERROR_SUCCESS) {
        if (dim > 0) {
            buf = new TCHAR[dim + 1];

            err = RegQueryValueEx(
                    key,
                    p,
                    NULL,
                    NULL,  // we currently support only strings
                    (UCHAR*)buf,
                    &dim
                    );            
        }
    }
    //else MessageBox(NULL,  T("Error"), T("getConfigParameter"), MB_OK);
    
    if (!buf) 
        buf = wstrdup(TEXT(""));

finally:
  
    if (key != 0) {
        RegCloseKey(key);
    }

    return buf;
}


/*
* return code -10: no source to sync
*/

DWORD synchronizationMailOnly () {
    
    wchar_t MAILS       [15];       
    int ret = 0;
    wcscpy(MAILS,         MAILS_NAME       );
    wchar_t t[12];
    wchar_t path [256];
    memset(path, 0, 256*sizeof(wchar_t));
    memset(t, 0, 12*sizeof(wchar_t));
    SyncSource** ssArray = new SyncSource*[2];
    TCHAR* msxmessagesize = getMailMaxMsgSize();
    unsigned long previousMaxMsgSize = 0;
    LogLevel ll = getLOGLevel();
    LOG.setLevel(ll);

    DMTClientConfig config(APPLICATION_URI); 
    config.read();    
    //SyncClient syncClient;
    CustomSyncClient syncClient;    

    int days = 0;
    int attachSize = 0;

    wchar_t* downloadAge = NULL;
    SourceFilter* filter = NULL;

    ClientSettings cs(APPLICATION_URI);
    cs.readConfig();
    //getClientConfigurationInternal (NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);
    string s = cs.getPath(); wchar_t* path2 = toWideChar(s.c_str());
    wcscpy(path, path2); delete [] path2;
    
    // mail
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");   
    MailSyncSourceConfig sc;    
    sc = m.getMailSourceConfig(true);    
    MailSyncSource mailSource = MailSyncSource (MAILS, &sc);    

    if (strcmp(sc.getSync(), "none") == 0) {
        ret = -10;        
        goto finally;
    }
    // check the refresh from server 
    if (strcmp(sc.getSync(), "refresh-server") == 0 || strcmp(sc.getSync(), "refresh") == 0) {
        
        int result = 0;
        result = checkRefreshFromServer(OL_MAIL);
        
        if (result == -1) {
            ret = -10;        
            goto finally;
        }
    }

    if (strcmp(sc.getSync(), "slow") == 0)
        SyncResultsLogger.syncTypes |= SYNCRESULTS_MAIL_SLOW;
    
    wchar_t* ssfs = getSourcesForSlow();
    sourceNameForSlow.push_back(ssfs);
    syncClient.namesOfSources = sourceNameForSlow;    
    syncSourceListener->sc = &syncClient;
    delete [] ssfs;
    resetSourceForSlow();
      
    //no more used with large object support
    //previousMaxMsgSize = config.getAccessConfig().getMaxMsgSize();
    //config.getAccessConfig().setMaxMsgSize(wcstol(msxmessagesize, NULL, 10));            

    getFolderToSync(sc, t);    
    
    days = sc.getDownloadAge();
    downloadAge = getDayBefore(days);
    attachSize = sc.getAttachSize();
    //
    // create a default filter related on max message size. This permits to not loose any mail
    // even if it appears to large
    //
    if (attachSize == -1) {
        //reserved value for the remain part of syncml msg
        // 1024 because the filter multiply for 1024
        attachSize = (wcstol(msxmessagesize, NULL, 10) - 32768) / 1024;  
    }

    filter = 
        ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(), attachSize);        
    
    //
    //
    // create filter as default

    mailSource.setPath(path);
    mailSource.setFolderToSync(t);
    mailSource.setFilter(filter);
      
    ssArray[0] = &mailSource;
    ssArray[1] = NULL;                
    
    
    ret = syncClient.sync(config, ssArray); 
    
    if (ret == 0) {
        //config.getAccessConfig().setMaxMsgSize(previousMaxMsgSize);
        config.save();           
    }    
    
    if (mailSource.getIsMailInOutbox() && ret == 0) {
        LOG.debug("There are mails in outbox");
        /*
        config.getAccessConfig().setMaxMsgSize(wcstol(msxmessagesize, NULL, 10));    
        sc = m.getMailSourceConfig(true); // refresh the config of the mailSyncSource
        ret = syncClient.sync(config, ssArray);

        if (ret == 0) {
            config.getAccessConfig().setMaxMsgSize(previousMaxMsgSize);
            config.save();            
        }
        */
    }
    
    if (mailSource.getIsMailInInbox()) {
        MessageBeep(MB_ICONASTERISK);
    }
    
    if (mailSource.getFailedSendMailInOutbox()) {
        //TimedMessageBox(NULL, OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
        //    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, OUT_MESSAGE_TIMEOUT*1000);
        MessageBox(NULL, ONE_OUT_MESSAGE_NOT_SENT, TEXT("Alert"),
            MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);

    }
    
    LOG.debug("Mail Sync Ended.");

finally:
    
    // TODO: send  to UI (if present)
    //if(HwndFunctions::wnd)
    //    SendMessage(HwndFunctions::wnd, (WM_APP+2), NULL,NULL);

    if (msxmessagesize) { delete [] msxmessagesize; msxmessagesize = NULL;}
    return ret;
}


DWORD checkStartSync() {
    
    DWORD pid = 0;
    // Try to create the mutex
    HANDLE hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );
    if(hMutex){
        if(GetLastError() == ERROR_ALREADY_EXISTS){
            pid = getProcessPid( TEXT("startsync.exe") );
        }
        CloseHandle( hMutex );
    }
    return pid;

#if 0
    DWORD out = 0;
    PROCESSENTRY32 lppe; 
    BOOL next = TRUE;
    HANDLE hProcess = 0;
    DWORD ret = 0;

    HANDLE hSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0); // 0 is however ignored
    
    if (hSnapshot == INVALID_HANDLE_VALUE) 
        return 0;
    
    lppe.dwSize = sizeof( PROCESSENTRY32 );

    if (!Process32First(hSnapshot, &lppe))
        return 0;

    if (wcsstr(lppe.szExeFile, TEXT("startsync.exe")) != NULL) {             
        ret = lppe.th32ProcessID;
        goto exit;
    }
    
    do {
        next = Process32Next(hSnapshot, &lppe);

        if (!next)
            break;
        
        if (wcsstr(lppe.szExeFile, TEXT("startsync.exe")) != NULL ) {                  
            ret = lppe.th32ProcessID; 
            goto exit;
        }

    } while(next);

exit:
    if (hSnapshot) 
        CloseToolhelp32Snapshot(hSnapshot);
    CloseHandle(hProcess);
    return ret;
#endif
}

//#define ASK_REFRESH_MESSAGE TEXT("All the local %s will be deleted before adding the server items. Are you sure you want to continue?")
#define ASK_REFRESH_MESSAGE TEXT("A sync to replace your local %s with your server %s has been requested. All the local %s will be deleted before adding the server items. Are you sure you want to continue?")
#define ASK_REFRESH_MESSAGE_MAILS TEXT("A sync to replace your local email with your mail server email has been requested. All the local mails will be deleted before adding the server items. Are you sure you want to continue?")
#define ASK_REFRESH_TIMEOUT 15

int checkRefreshFromServer(int type) {
    /*
    OL_CONTACTS     = 10,
    OL_CALENDAR     = 9,
    OL_TASK         = 13,
    OL_BRIEFCASE    = 600,
    OL_NOTES        = 601,
    OL_FAVORITES    = 602
    OL_MAIL         = 610
    */
    wchar_t message[256];
    
    wchar_t sourceNameW [128];       
    int ret = 0;
    message[0] = 0;  
    sourceNameW[0] = 0;

    if (type== OL_CONTACTS)
        wcscpy(sourceNameW, TEXT("contacts"));
    else if (type == OL_CALENDAR)
        wcscpy(sourceNameW, TEXT("calendars"));
    else if (type == OL_TASK)
        wcscpy(sourceNameW, TEXT("tasks"));
    else if (type == OL_NOTES)
        wcscpy(sourceNameW, TEXT("notes"));
    else if (type == OL_BRIEFCASE)
        wcscpy(sourceNameW, TEXT("files"));
    //else if (type == OL_MAIL)
    //    wcscpy(sourceNameW, TEXT("mails"));
    
    if(type == OL_MAIL)
        wcscpy(message, ASK_REFRESH_MESSAGE_MAILS);    
    else
        wsprintf(message, ASK_REFRESH_MESSAGE, sourceNameW, sourceNameW, sourceNameW);
    
    HwndFunctions::closePreviousMsgBox();
    int val = TimedMessageBox(NULL, message, TEXT("Funambol Alert"),
                         MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

    if (val == IDYES) {
        LOG.debug("User said yes to delete local %S and refresh.", sourceNameW);            
    }
    else {
        LOG.debug("User said no to delete local %S and refresh." , sourceNameW);     
        ret = -1;
    }
   
    return ret;    
}





/*
 * get the parameter sourcesForSlow that contains the list name of the sources
 */

wchar_t* getSourcesForSlow() {
    HKEY key = NULL;    
    DWORD res;    
    long err = 0;
    TCHAR *p = TEXT("sourcesForSlow");
    ULONG dim = 0;
    TCHAR *buf = NULL;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            TEXT("\\Software\\Funambol\\SyncclientPIM"),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        sprintf(lastErrorMsg, "Invalid context path: %s", "Software/Funambol/SyncclientPIM/spds/sources/mails");
        goto finally;
    }

    // Get value length
    err = RegQueryValueEx(
            key,
            p,
            NULL,
            NULL,  // we currently support only strings
            NULL,
            &dim
            );

    if (err == ERROR_SUCCESS) {
        if (dim > 0) {
            buf = new TCHAR[dim + 1];

            err = RegQueryValueEx(
                    key,
                    p,
                    NULL,
                    NULL,  // we currently support only strings
                    (UCHAR*)buf,
                    &dim
                    );            
        }
    }   
    
    if (!buf) 
        buf = wstrdup(TEXT(""));

finally:
  
    if (key != 0) {
        RegCloseKey(key);
    }

    return buf;
}

/*
* reset the sourcesForSlow parameter.
*/
void resetSourceForSlow() {
    

    HKEY key;
    DWORD res;
    wchar_t *p = TEXT("sourcesForSlow");;    
    
    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            TEXT("\\Software\\Funambol\\SyncclientPIM"),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );
       

    RegSetValueEx(
            key,
            p,
            NULL,
            REG_SZ,  // we currently support only strings
            (UCHAR*)TEXT(""),
            (wcslen(TEXT(""))+1)*sizeof(WCHAR)
            );
        
    if (key != 0) {
        RegCloseKey(key);
    }
} 

/*
* To get the complete mail id. The AdviseSink of the syncml transport returns an array of /id1, /id2
* But the complete id mapped with the server is I/id1, S/id2. So to ask the right mail is necessary to retrieve
* the complete id that is stored in the files containing the cached situation
*/
wchar_t* findMailIdInFile(const wchar_t* fileName, const wchar_t* partialID) {
                                                  
    wchar_t* idComplete = NULL;
    wchar_t line[512];
    FILE* f = _wfopen(fileName, TEXT("r"));
        
    if (f == NULL)
        return NULL;
       
    while(fgetws(line, 511, f) != NULL) {
                        
        if (wcsstr(line, partialID ) != NULL) {            
            if (idComplete = getElementContent(line, TEXT("id"), 0))
                break;

        }        
    }    
    fclose(f);
    return idComplete;
}

wchar_t* completeMailId(const wchar_t* path, const wchar_t* partialID) {

    bool isLast = false;
    wchar_t fileName[255];
    wchar_t* idComplete = NULL;
    
    WIN32_FIND_DATA FileData;
    wchar_t filter[255];
    wsprintf(filter, TEXT("%s\\mails_*"), path);
    HANDLE hFind = FindFirstFile(filter, &FileData);
    
    if (hFind == INVALID_HANDLE_VALUE) {
        LOG.debug("No mails_ file found.");
        return NULL;
    }
    
    while (!isLast) {
        wsprintf(fileName, TEXT("%s\\%s"), path, FileData.cFileName);                                           
        idComplete = findMailIdInFile(fileName, partialID); 
        
        if (idComplete) { // id completed found
            break;            
        } else {
            if (!FindNextFile(hFind, &FileData))                    
                isLast = TRUE;            
        }        
    }
    
    FindClose(hFind);
    return idComplete;

}

/*
* Return the number of email that are not sent in outbox folder
* When the connector will send the messate cause for every item
* we can get the message too
*/

wstring getMailUnsent(MailSyncSource mailSource) {
    
    wstring messageToRet = L"";
    SyncSourceReport* rep = mailSource.getReport();
    int res = rep->getItemReportFailedCount(SERVER, COMMAND_ADD);  
    int number = 0, i = 0;
    BOOL found = FALSE;
    for (i = 0; i < res; i++) {
        ItemReport* r = rep->getItemReport(SERVER, COMMAND_ADD, i);
        if (wcsstr(r->getId(), L"O/") != 0 && (r->getStatus() == 500 || r->getStatus() == 418 || r->getStatus() == 419)) {
            found = TRUE;
            if (r->getStatusMessage()) {
                LOG.debug("The outbox mail (id: %S) cannot be sent. Server says: %S", r->getId(), r->getStatusMessage());
                messageToRet = r->getStatusMessage();
            }
            number++;
        }
    }

    if (!found) {
        res = rep->getItemReportFailedCount(SERVER, COMMAND_REPLACE);  
        for (i = 0; i < res; i++) {
            ItemReport* r = rep->getItemReport(SERVER, COMMAND_REPLACE, i);
            if (wcscmp(r->getId(), L"O/") != 0 && (r->getStatus() == 500 || r->getStatus() == 418 || r->getStatus() == 419)) {
                if (r->getStatusMessage()) {
                    LOG.debug("The outbox mail (id: %S) cannot be sent. Server says: %S", r->getId(), r->getStatusMessage());
                    messageToRet = r->getStatusMessage();
                }
                number++;
            }
        }
        
    }

    if (number == 1) {
        /*
        * Added extension to the name if there isn't one
        */ 
        wstring msg(messageToRet);
        std::transform(messageToRet.begin(), messageToRet.end(), msg.begin(), tolower);      

        if (messageToRet == L"" || (msg.find(L"error sending") == wstring::npos && 
                                    msg.find(L"error adding") == wstring::npos) 
                                ||  msg.find(L"com.funambol") != wstring::npos ) {
            messageToRet = ONE_OUT_MESSAGE_NOT_SENT;
        } else {
            messageToRet.insert(0, OUT_MESSAGE_NOT_SENT);
        }
    } else {
        messageToRet = MORE_OUT_MESSAGE_NOT_SENT;
    }

    return messageToRet;
}
